------------------------------------------------------------
-- Bombe alain
------------------------------------------------------------

-- Fns
function aBombe_Create(toGroup, forcedPos, missileLevel)
	if forcedPos == nil then
		forcedPos = Vector2D(math.random() * screenSize.x, -50)
	end
	
	local enemy = stockABombe:CloneToGroup(toGroup)
	enemy:SetPosition(forcedPos)
	
	-- Speed
	enemy:SetSpeed(enemy:GetSpeed() * (0.8 + math.random() * 0.4)) -- Speed +- 40%
	
	-- Info
	missileLevel = math.min(missileLevel, #bombeATexTbl)
	missileLevel = math.max(missileLevel, 1)
	
	enemy:FindByNameSprite("spr"):SetImage(bombeATexTbl[missileLevel])
	enemy:SetUserData("life", 35 + missileLevel * 30)
	enemy:SetUserData("damage", 20 + missileLevel * 20)
	enemy:SetUserData("points", missileLevel)
	
	return enemy
end

function aBombe_OnHit(phBullet, phEnemy, hitPos)
	enemyGrp = phEnemy:GetParent()
	bulletGrp = phBullet:GetParent()
	
	enemyGrp:SetUserData("life", enemyGrp:GetUserData("life") - bulletGrp:GetUserData("damage"))

	if enemyGrp:GetUserData("life") <= 0 then -- If the enemy is dead
		-- Enemy fn
		-- Explosion
		CreateExplosion(hitPos, math.pi * 3/5, bulletGrp:GetSpeed())

		-- Play Snd
		gameScreen:PlaySound(sndEnemyBurstTbl[math.ceil(#sndEnemyBurstTbl * math.random())])
		
		-- Points
		score = score + enemyGrp:GetUserData("points")
		
		enemyGrp:Destroy() -- We destroy the group
	else -- The enemy is style alive
		gameScreen:PlaySound(sndEnemyResist)
		
		-- Shield
		CreateSpriteFadeOutAnim(enemyGrp, bombeAShieldTex:GetSize() / 2 * -1, bombeAShieldTex, 1)
	end
	
	if phBullet:GetName() ~= "phLaser" then
		phBullet:GetParent():Destroy() -- We destroy the group
	end
end

function aBombe_OnShieldShock(phShield, phEnemy, hitPos)
	enemyGrp = phEnemy:GetParent()
	
	-- Damage
	ChangeLife(- enemyGrp:GetUserData("damage"))
	CreateShieldImpact(hitPos.x)
	
	-- Explosion
	CreateExplosion(hitPos, math.pi, Vector2D(0, -1))
	gameScreen:PlaySound(sndShieldResist)
	
	-- We destroy the group
	enemyGrp:Destroy()
end

-- Save
addToSpawnList(function(toGroup, forcedPos) aBombe_Create(toGroup, forcedPos, 1); end, 6, 1.5)
addToSpawnList(function(toGroup, forcedPos) aBombe_Create(toGroup, forcedPos, 2); end, 13, 1.5)
addToSpawnList(function(toGroup, forcedPos) aBombe_Create(toGroup, forcedPos, 3); end, 20, 2)

-- Missile
stockABombe = stockScreen:CreateGroup("enemyaBombe")
stockABombe:SetSpeed(Vector2D(0, 60))
stockABombe:SetUserData("enemyType", "missile_a")

-- Functions
stockABombe:SetUserData("OnHitFn", aBombe_OnHit)
stockABombe:SetUserData("OnShieldShockFn", aBombe_OnShieldShock)

-- Sprite
tmpSprite = stockABombe:CreateSprite("spr")
tmpSprite:SetImage(bombeATexTbl[1])
tmpSprite.y = - tmpSprite:GetSize().y / 2
tmpSprite.x = - tmpSprite:GetSize().x / 2

-- Physic
--stockABombe:CreatePhTriangle(phGroupEnemies, "ph", Vector2D(- tmpSprite:GetSize().x / 2, - tmpSprite:GetSize().y / 2), Vector2D(tmpSprite:GetSize().x / 2, - tmpSprite:GetSize().y / 2), Vector2D(0, tmpSprite:GetSize().y / 2))

--stockABombe:CreatePhCircle(phGroupEnemies, "ph", Vector2D(0, 0), tmpSprite:GetSize().x / 2)
local ph = stockABombe:CreatePhRect(phGroupEnemies, "ph", Vector2D(tmpSprite:GetSize().x, tmpSprite:GetSize().y))
ph:SetPosition(Vector2D(- tmpSprite:GetSize().x / 2, - tmpSprite:GetSize().y / 2))
