------------------------------------------------------------
-- Plane 1
------------------------------------------------------------

-- Create
function plane1_Create(toGroup, forcedPos, enemyLevel)
	local enemy = stockPlane1:CloneToGroup(toGroup)
	
	local tmpSpr = enemy:FindByNameSprite("spr")
	
	-- Speed
	
	-- Direction
	local negRatio = 1;
	if math.random() > 0.5 then
		negRatio = -1;
	end
	enemy:SetSpeed(Vector2D((90 + math.random() * 100) * negRatio, -5 + math.random() * 10))
	tmpSpr:FlipX(enemy:GetSpeed().x > 0)
	
	if forcedPos == nil then
		if enemy:GetSpeed().x > 0 then -- Gauche à droite
			forcedPos = Vector2D(-10 - tmpSpr:GetSize().x, 50 + math.random() * 200)
		else
			forcedPos = Vector2D(screenSize.x + 10 + tmpSpr:GetSize().x, 50 + math.random() * 200)
		end
	end
	
	if enemy:GetSpeed().x < 0 then -- Invertion particules
		local tmp = enemy:FindByNameParticleSource("particle")
		tmp:SetPositionX(enemy:FindByName("spr"):GetSize().x / 2 - 3)
		tmp:SetEmitterDirection(Vector2D(1, 0))
	end
	
	-- Pos
	enemy:SetPosition(forcedPos)
	
	-- Level
	local texTbl = planeTexTbl;
	
	enemyLevel = math.min(enemyLevel, #texTbl)
	enemyLevel = math.max(enemyLevel, 1)
	
	enemy:FindByNameSprite("spr"):SetImage(texTbl[enemyLevel])
	
	-- Timer
	local event = gameScreen:RegisterTimer(plane1_OnTimer, 5 - enemyLevel - 0.5 + math.random() * 1, true)
	guiLib_addAutoDestroyEvent(enemy, event)
	event:SetUserData("group", enemy)
	event:SetUserData("enemyLevel", enemyLevel)

	-- Return
	return enemy
end

function plane1_OnHit(phBullet, phEnemy, hitPos)
	enemyGrp = phEnemy:GetParent()
	bulletGrp = phBullet:GetParent()
	
	enemyGrp:SetUserData("life", enemyGrp:GetUserData("life") - bulletGrp:GetUserData("damage"))

	if enemyGrp:GetUserData("life") <= 0 then -- If the enemy is dead
		-- Enemy fn
		-- Explosion
		CreateExplosion(hitPos, math.pi * 3/5, bulletGrp:GetSpeed())

		-- Play Snd
		gameScreen:PlaySound(sndEnemyBurstTbl[math.ceil(#sndEnemyBurstTbl * math.random())])
		
		-- Points
		score = score + enemyGrp:GetUserData("points")
		
		enemyGrp:Destroy() -- We destroy the group
	else -- The enemy is style alive
		gameScreen:PlaySound(sndEnemyResist)
	end
	
	if phBullet:GetName() ~= "phLaser" then
		phBullet:GetParent():Destroy() -- We destroy the group
	end
end

function plane1_OnShieldShock(phShield, phEnemy, hitPos)
	enemyGrp = phEnemy:GetParent()
	
	-- Damage
	ChangeLife(- enemyGrp:GetUserData("damage"))
	
	-- Explosion
	CreateExplosion(hitPos, math.pi, Vector2D(0, -1))
	gameScreen:PlaySound(sndShieldResist)
	
	-- We destroy the group
	enemyGrp:Destroy()
end

-- Lunch missile
function plane1_OnTimer(event)
	local enemy = event:GetUserData("group")
	local enemyLevel = event:GetUserData("enemyLevel")
	
	-- Create a nuclear bomb
	nuclearShell_Create(enemy:GetParent(), enemy:GetPosition(), enemyLevel)
	
	-- Make the plane on top (refresh lowZPos)
	enemy:SetZpos(enemy:GetZpos())
end

-- Register
addToSpawnList(function(toGroup, forcedPos) plane1_Create(toGroup, forcedPos, 1); end, 3, 0.7)
addToSpawnList(function(toGroup, forcedPos) plane1_Create(toGroup, forcedPos, 2); end, 7, 1.5)
addToSpawnList(function(toGroup, forcedPos) plane1_Create(toGroup, forcedPos, 3); end, 15, 2)

-- Plane
stockPlane1 = stockScreen:CreateGroup("plane1")
stockPlane1:SetSpeed(Vector2D(300, 0))
stockPlane1:SetUserData("life", 5)
stockPlane1:SetUserData("damage", 0)
stockPlane1:SetUserData("points", 2)
stockPlane1:SetUserData("enemyType", "plane1")

-- Functions
stockPlane1:SetUserData("OnHitFn", plane1_OnHit)
stockPlane1:SetUserData("OnShieldShockFn", plane1_OnShieldShock)

-- Sprite
local tmpSprite = stockPlane1:CreateSprite("spr")
tmpSprite:SetImage(planeTexTbl[1])
tmpSprite:SetPosition(Vector2D(- tmpSprite:GetSize().x / 2, 7 - tmpSprite:GetSize().y))

-- Physic
stockPlane1:CreatePhLine(phGroupEnemies, "ph", Vector2D(- tmpSprite:GetSize().x / 2, 0), Vector2D(tmpSprite:GetSize().x / 2, 0))

-- Reacteur --
model = ParticleModel(0.2, 0.4)
model:AddMutableParam("alpha", 0.7, 0)
model:AddMutableRandomParam("blue", 0.7, 1, 0, 0.1)
model:AddMutableRandomParam("red", 0.5, 0.7, 0.7, 1)
model:AddMutableRandomParam("green", 0.2, 0.5, 0, 0.2)
model:AddMutableRandomParam("size", 0.5, 0.7, 1.2, 1.8)

emitter = ParticleEmitter()
emitter:MakeDisk(Vector2D(0, 0), 4)
emitter:SetAngles(3.14 / 4, 0)
emitter:SetDirection(Vector2D(-1, 0))

-- Particle
particleObj = stockPlane1:CreateParticleSource("particle", model, 12)
particleObj:MakeRenderImages(texParticlePoint, Vector2D(40, 40), true)
particleObj:SetEmitter(emitter)
particleObj:SetFlow(30)
particleObj:SetForce(30, 70)
particleObj:SetFriction(3)
particleObj:SetGravity(Vector2D(0, 0))
particleObj.position = Vector2D(- tmpSprite:GetSize().x / 2 + 3, 0)
particleObj.zPos = -100
